# C++ 中的智能指针

> 原文：<https://www.javatpoint.com/smart-pointers-in-cpp>

指针用于存储另一个变量的地址。换句话说，指针提取程序外部资源的信息(堆内存)。

我们使用资源的副本，并且要进行更改，它是在复制的版本中完成的。要更改原始资源的内容，可以使用智能指针。

### 普通指针面临的问题

下面是一个描述普通指针问题的例子。

创建了具有两个数据成员(*长度和宽度*)的类**矩形**。一个函数 **fun()** 动态创建一个矩形的对象。

当 fun()函数结束时，对象 p 被销毁。因为我们没有删除 p，所以内存仍然是已分配的，并且这个已分配的资源将不可用于其他变量。

在 **main()中，**我们执行一个无限循环，这个循环将继续创建 p 个对象并分配资源。这个问题变成了一个**内存泄漏**，智能指针是一个解决方案。

**代码**

```

#include using namespace std;

class Rectangle { // created a class Rectangle
private:
	int length; // data member as length of rectangle
	int breadth; // data member as breadth of rectangle

};

void fun() // the function to indicate the problem with normal pointer 
{

	Rectangle* p = new Rectangle(); // Create a dynamic object p 
}

int main()
{
	// Infinite Loop
	while (1) { // Run an infinite loop that will allocate p 
		fun();
	}
} 
```

#### 注意——像 JAVA、C#这样的语言有自己的垃圾收集器，可以智能地释放未使用的内存资源。

## 智能指针

我们将实现智能指针，这样它们将释放未使用资源的内存。

用指针、重载运算符(->、*)和析构函数创建一个类。

当析构函数的对象超出作用域时，析构函数将被自动调用，动态分配的内存将被自动删除。

**例**

```

#include using namespace std;

class SmartPtr { // Create the class to implement smart Pointer
    int* ptr; // Actual pointer
public:
// Create an explicit constructor 
    explicit SmartPtr(int* p = NULL) { ptr = p; }

    // Destructor to deallocate the resource used 
    ~SmartPtr() { delete (ptr); }

    // Overloading dereferencing operator
    int& operator*() { return *ptr; }
};

int main()
{
    SmartPtr ptr(new int());
    *ptr = 100;
    cout << *ptr;

    // We don't need to call delete ptr: when the object
    // ptr goes out of scope, the destructor for it is automatically
    // called, and destructor does delete ptr.

    return 0;
} 
```

**输出**

```
100

```

以上示例仅适用于 *int。*我们将创建一个适用于每种数据类型的模板。

**代码**

```

#include using namespace std;

template <class t="">// Create a template class 
class SmartPtr {
	T* ptr; // Actual pointer
public:
	// Constructor
	explicit SmartPtr(T* p = NULL) { ptr = p; }

	// Destructor
	~SmartPtr() { delete (ptr); }

	// Overloading dereferncing operator
	T& operator*() { return *ptr; }

	// Overloading arrow operator so that
	// members of T can be accessed
	// like a pointer (useful if T represents
	// a class or struct or union type)
	T* operator->() { return ptr; }
};

int main()
{
	SmartPtr <int>ptr(new int());
	*ptr = 100;
	cout << *ptr;
	return 0;
}</int></class> 
```

**输出**

```
100

```

## 智能指针的类型

*   unique _ ptr

这种类型的对象只存储一个对象。要分配不同的对象，当前对象将被解除分配。

**代码**

```

#include #include <memory>using namespace std;

class Rectangle { // Create the class
    // Data members
    int length; // length of rectangle 
    int breadth; // breadth of rectangle 

public:
    Rectangle(int l, int b)
    { // parameterised constructor
        length = l;
        breadth = b;
    }

    int area()
    { // calculate area
        return length * breadth; // return the area 
    }
};

int main()
{

    unique_ptr <rectangle>P1(new Rectangle(20, 5));
    cout << P1->area() << endl; // This'll print 100

    // unique_ptr <rectangle>P2(P1);
    unique_ptr <rectangle>P2;
    P2 = move(P1);

    // This'll print 100
    cout << P2->area() << endl;

    return 0;
}</rectangle></rectangle></rectangle></memory> 
```

**输出**

```
100
100 

```

*   **Shared_ptr**

在 shared_ptr 中，多个对象可以同时指向一个指针。使用 ***use_count()*** 方法维护用于表示对象的参考计数器。

**代码**

```

#include #include <memory>using namespace std;

class Rectangle { // Create the class
    // Data members
    int length; // length of rectangle 
    int breadth; // breadth of rectangle 

public:
    Rectangle(int l, int b)
    { // parameterised constructor
        length = l; 
        breadth = b;
    }

    int area()
    { // calculate area of rectangle 
        return length * breadth; // return area 
    }
};

int main()
{

    shared_ptr <rectangle>P1(new Rectangle(20, 5)); // create shared //ptr P1
    // This'll print 100
    cout << P1->area() << endl;
	// Create shared ptr P2
    shared_ptr <rectangle>P2;
    P2 = P1;

    // This'll print 100
    cout << P2->area() << endl;

    // This'll now not give an error,
    cout << P1->area() << endl; // prints 100

// reference counter of P2 is 2   
    cout << P1.use_count() << endl; // prints 2 
    return 0;
}</rectangle></rectangle></memory> 
```

**输出**

```
100
100
100
2

```

*   **弱 _ptr**

Weak_ptr 类似于共享指针。不同之处在于，它不维护引用计数器，并且指针上没有对象的强保持。此属性可能会导致死锁，因为不同的对象将尝试保存指针。

* * *