# Rust特性

> 原文：<https://www.javatpoint.com/rust-trait>

*   Rust trait 是 Rust 语言的一个特性，描述了它能提供的每种类型的功能。
*   一个特性类似于在其他语言中定义的接口特性。
*   特性是对方法签名进行分组以定义一组行为的一种方式。
*   特征是通过使用特征关键字来定义的。

**性状的句法:**

```

 trait trait_name
{
     //body of the trait.
}  

```

在上面的例子中，我们声明了特征，后跟特征名。在花括号内，方法签名被声明来描述实现特征的类型的行为。

**我们来看一个简单的例子:**

```

 struct Triangle
{
  base : f64,
  height : f64,
}
trait HasArea
{
  fn area(&self)->f64;
}

impl HasArea for Triangle
{
  fn area(&self)->f64
  {
    0.5*(self.base*self.height)
  }
}
fn main()
{
  let a = Triangle{base:10.5,height:17.4};
  let triangle_area = a.area();
  println!("Area of a triangle is {}",triangle_area); 
}

```

**输出:**

```
Area of a triangle is 91.35

```

在上面的例子中，声明了一个名为 HasArea 的特性，它包含了 Area()函数的声明。HasArea 是在三角形类型上实现的。通过使用结构的实例，即 a.area()简单地调用 area()函数。

## 作为论据的特性

特征也可以用作许多不同类型的参数。

上面的例子实现了 HasArea 特性，它包含了 Area()函数的定义。我们可以定义调用 area()函数的 calculate_area()函数，并且使用实现 HasArea 特性的类型的实例来调用 area()函数。

**我们来看看语法:**

```

 fn calculate_area(item : impl HasArea)
{
   println!("Area of the triangle is : {}",item.area());
 } 

```

## 泛型函数的特征界限

特征很有用，因为它们描述了不同方法的行为。但是，泛型函数不遵循这个约束。**让我们通过一个简单的场景来理解这一点:**

```

 fn calculate_area<T>( item : T)
{
    println!(?Area of a triangle is {}?, item.area());
}

```

在上述情况下，Rust 编译器抛出一个“找不到类型为 T 的名为的方法”的错误。如果我们把这个特征和一般的 T 联系起来，那么下面的错误是可以克服的:

```

 fn calculate_area<T : HasArea> (item : T)
{
println!("Area of a triangle is {} ",item.area());

}

```

在上述情况下，<t: hasarea="">表示“T 可以是实现 HasArea 特性的任何类型”。Rust 编译器知道任何实现 HasArea 特性的类型都有一个 Area()函数。</t:>

**我们来看一个简单的例子:**

```

 trait HasArea
{
  fn area(&self)->f64;
}
struct Triangle
{
  base : f64,
  height : f64,
}

impl HasArea for Triangle
{
  fn area(&self)->f64
  {
    0.5*(self.base*self.height)
  }
}
struct Square
{
  side : f64,
}

impl HasArea for Square
{
  fn area(&self)->f64
  {
     self.side*self.side
  }
}
fn calculate_area<T : HasArea>(item : T)
{
  println!("Area is : {}",item.area());
}

fn main()
{
  let a = Triangle{base:10.5,height:17.4};
  let b = Square{side : 4.5};
  calculate_area(a);
  calculate_area(b);
}

```

**输出:**

```
Area is : 91.35
Area is : 20.25

```

在上面的例子中，calculate_area()函数在“T”上是通用的。

## 实施特征的规则

实现这一特性有两个限制:

*   如果特性没有在您的范围内定义，那么它就不能在任何数据类型上实现。

**我们来看一个简单的例子:**

```

 use::std::fs::File;
fn main()
{
  let mut f = File::create("hello.txt");
  let str = "javaTpoint";
  let result = f.write(str);
}

```

**输出:**

```
error : no method named 'write' found.
           let result = f.write(str);

```

**In the above case, Rust compiler throws an error, i.e., "no method named 'write' found" as** use::std::fs::File; namespace does not contain the write() method. Therefore, we need to **use the Write trait to remove the compilation error**.

*   我们正在实施的特性必须由我们来定义。例如:如果我们定义**有一个区域**特性，那么我们可以为类型 i32 实现这个特性。然而，我们无法对 i32 型实施由 Rust 定义的**转向**特性，因为该类型和特性都没有在我们的机箱中定义。

## 多性状界限

*   **使用“+”运算符。**

如果我们想绑定多个特征，我们使用+运算符。

**我们来看一个简单的例子:**

```

 use std::fmt::{Debug, Display};
fn compare_prints<T: Debug + Display>(t: &T)
 {
 println!("Debug: '{:?}'", t);
 println!("Display: '{}'", t);
}

fn main() {
    let string = "javaTpoint";
    compare_prints(&string);
    }

```

**输出:**

```
Debug: ' "javaTpoint"'
Display: ' javaTpoint'

```

在上面的示例中，显示和调试特征通过使用“+”运算符绑定到类型“T”。

*   **使用‘where’子句。**
    *   可以使用出现在左括号“{ 0 }”前面的“where”子句编写绑定。
    *   “where”子句也可以应用于任意类型。
    *   当使用“where”子句时，它会使语法比正常语法更具表现力。

**我们来看看:**

```

 fn fun<T: Display+Debug, V: Clone+Debug>(t:T,v:V)->i32
{
     //block of code;
} 

```

当‘where’用于上述情况时:

```

 fn fun<T, V>(t:T, v:V)->i32
   where T : Display+ Debug, 
              V : Clone+ Debug
{
        //block of code;
}

```

在上面的例子中，我们使用了 where 子句的第二种情况使程序更具表达性和可读性。

**我们来看一个简单的例子:**

```

 trait Perimeter
{
  fn a(&self)->f64;
}
struct Square
{
  side : f64,
}
impl Perimeter for Square
{
  fn a(&self)->f64
  {
    4.0*self.side
  }
}
struct Rectangle
{
 length : f64,
 breadth : f64,
}
impl Perimeter for Rectangle

{
 fn a(&self)->f64
 {
   2.0*(self.length+self.breadth)
 }
}
fn print_perimeter<Square,Rectangle>(s:Square,r:Rectangle)
  where Square : Perimeter,
        Rectangle : Perimeter
		{
		  let r1 = s.a();
		  let r2 = r.a();
		  println!("Perimeter of a square is {}",r1);
		  println!("Perimeter of a rectangle is {}",r2);
		}
		fn main()
		{
		  let sq = Square{side : 6.2};
		  let rect = Rectangle{length : 3.2,breadth:5.6};
		  print_perimeter(sq,rect);
		}

```

**输出:**

```
Perimeter of a square is 24.8
Perimeter of a rectangle is 17.6

```

## 默认方法

如果方法的定义是已知的，可以将默认方法添加到特征定义中。

**我们来看看:**

```

 trait Sample
{ 
   fn a(&self);
   fn b(&self)
   {
       println!("Print b");
   } 
} 

```

在上面的例子中，默认行为被添加到特征定义中。我们也可以覆盖默认行为。**让我们通过一个例子来看看这个场景:**

```

 trait Sample
{
 fn a(&self);
 fn b(&self)
 {
   println!("Print b");
 } 
} 

struct Example
{
 a:i32,
 b:i32,
}

impl Sample for Example
{
  fn a(&self)
  {
    println!("Value of a is {}",self.a);
  }

  fn b(&self)
  {
    println!("Value of b is {}",self.b);
  }
}
fn main()
{
  let r = Example{a:5,b:7};
  r.a();
  r.b();  
}

```

**输出:**

```
Value of a is : 5
Value of b is : 7

```

在上面的例子中，b()函数的行为是在特质被覆盖时定义的。因此，我们可以得出结论，我们可以覆盖特性中定义的方法。

## 遗产

从另一种特性衍生出来的特性称为遗传。有时候，实现一个特征就需要实现另一个特征。如果我们想从“A”特质中衍生出“B”特质，那么它看起来就像:

```

 trait B : A;

```

**我们来看一个简单的例子:**

```

 trait A
{
  fn f(&self);
}
trait B : A
{
  fn t(&self);
}
struct Example
{
  first : String,
  second : String,
}
impl A for Example
{
  fn f(&self)
  {

   print!("{} ",self.first);
  }

 }
 impl B for Example
 {
  fn t(&self)
  {
    print!("{}",self.second);
  }
}
fn main()
{
  let s = Example{first:String::from("javaTpoint"),second:String::from("tutorial")};
  s.f();
  s.t();
}

```

**输出:**

```
javaTpoint tutorial

```

在上面的例子中，我们的程序正在实现‘B’特性。因此，它也要求实现‘A’特质。如果我们的程序没有实现‘A’特性，那么 Rust 编译器会抛出一个错误。

* * *