# Verilog阻塞和非阻塞

> 原文：<https://www.javatpoint.com/verilog-blocking-and-non-blocking>

Verilog 支持始终块中具有不同行为的阻塞和非阻塞赋值语句。

阻塞赋值类似于大多数流行编程语言中的软件赋值语句。非阻塞赋值是描述许多硬件系统的更自然的赋值语句，特别是对于综合而言。

分块赋值只能在少数情况下使用，如组合逻辑建模、定义函数或实现测试平台算法。所有符合 IEEE P1364.1 的合成工具都需要支持显式代码中的阻塞和非阻塞赋值，限制是每个变量和每个块只能使用一种或另一种赋值。

### 阻塞分配

阻塞赋值语句使用(=)运算符赋值，并在过程块中一个接一个地执行。但是，它不会阻止在并行块中运行的语句的执行。

```

module Block;
  reg [7:0] a, b, c, d, e;

  initial begin
    a = 8'hDA;
    $display ("[%0t] a=0x%0h b=0x%0h c=0x%0h", $time, a, b, c);
    b = 8'hF1;
    $display ("[%0t] a=0x%0h b=0x%0h c=0x%0h", $time, a, b, c);
    c = 8'h30;
    $display ("[%0t] a=0x%0h b=0x%0h c=0x%0h", $time, a, b, c);
  end

  initial begin
    d = 8'hAA;
    $display ("[%0t] d=0x%0h e=0x%0h", $time, d, e);
    e = 8'h55;
    $display ("[%0t] d=0x%0h e=0x%0h", $time, d, e);
  end
endmodule

```

有两个*块并行执行。语句在每个块中按顺序执行，两个块都在时间 0n 结束。*

 *更具体地说，首先分配变量，然后是 display 语句，然后是所有其他语句。

这在输出中是可见的，其中变量 *b* 和 *c* 在第一个显示语句中是 8'hxx。这是因为调用第一个$ ***显示*** 时，变量 *b* 和 *c* 赋值尚未执行。

```

ncsim> run
[0] a=0xda b=0xx c=0xx
[0] a=0xda b=0xf1 c=0xx
[0] a=0xda b=0xf1 c=0x30
[0] d=0xaa e=0xx
[0] d=0xaa e=0x55
ncsim: *W,RNQUIE: Simulation is complete.

```

在下面的示例中，我们将在同一组语句中添加一些延迟，以查看它的反应和行为。

```

module Block;
  reg [7:0] a, b, c, d, e;
  initial begin
    a = 8'hDA;
    $display ("[%0t] a=0x%0h b=0x%0h c=0x%0h", $time, a, b, c);
    #10 b = 8'hF1;
    $display ("[%0t] a=0x%0h b=0x%0h c=0x%0h", $time, a, b, c);
    c = 8'h30;
    $display ("[%0t] a=0x%0h b=0x%0h c=0x%0h", $time, a, b, c);
  end
  initial begin
    #5 d = 8'hAA;
    $display ("[%0t] d=0x%0h e=0x%0h", $time, d, e);
 	#5 e = 8'h55;
    $display ("[%0t] d=0x%0h e=0x%0h", $time, d, e);
  end
endmodule

```

执行后，它给出以下数据。

```

ncsim> run
[0] a=0xda b=0xx c=0xx
[5] d=0xaa e=0xx
[10] a=0xda b=0xf1 c=0xx
[10] a=0xda b=0xf1 c=0x30
[10] d=0xaa e=0x55
ncsim: *W,RNQUIE: Simulation is complete.

```

### 非阻塞分配

允许调度非阻塞赋值语句，而不阻塞以下语句的执行，并由(< =)符号指定。

相同的符号在表达式中用作关系运算符，在非阻塞赋值的上下文中用作赋值运算符。

举一个与上面相同的例子，用非阻塞赋值运算符(< =)替换所有(=)符号，我们将得到输出中的差异。

```

module Block;
  reg [7:0] a, b, c, d, e;
  initial begin
    a <= 8'hDA;
    $display ("[%0t] a=0x%0h b=0x%0h c=0x%0h", $time, a, b, c);
    b <= 8'hF1;
    $display ("[%0t] a=0x%0h b=0x%0h c=0x%0h", $time, a, b, c);
    c <= 8'h30;
    $display ("[%0t] a=0x%0h b=0x%0h c=0x%0h", $time, a, b, c);
  end
initial begin
    d <= 8'hAA;
    $display ("[%0t] d=0x%0h e=0x%0h", $time, d, e);
 	e <= 8'h55;
    $display ("[%0t] d=0x%0h e=0x%0h", $time, d, e);
  end
endmodule

```

现在，所有的$ ***显示*** 报表打印 ***'h'x*** 。这种行为的原因是非阻塞赋值语句的执行。

特定时间步长的每个非阻塞语句的 RHS 被捕获，并转移到下一个语句。

捕获的 RHS 值仅在时间步长结束时分配给 LHS 变量。

```

ncsim> run
[0] a=0xx b=0xx c=0xx
[0] a=0xx b=0xx c=0xx
[0] a=0xx b=0xx c=0xx
[0] d=0xx e=0xx
[0] d=0xx e=0xx
ncsim: *W,RNQUIE: Simulation is complete.

```

* * **