# Verilog控制块

> 原文：<https://www.javatpoint.com/verilog-control-blocks>

没有条件语句和其他控制逻辑流的方法，硬件行为就无法实现。Verilog 有一套机制和控制流块。

### 语句

这个条件语句用于决定是否执行某些语句。它非常类似于 C 语言中的 if-else-if 语句。如果表达式的计算结果为真，那么将执行第一条语句。

如果表达式的计算结果为 false，并且存在其他部分，则将执行其他部分。

**语法**

以下是 if-else-if 条件语句最简化的语法:

```

// if statement without else part
if (expression)
	[statement]

// if statement with an else part
if (expression)
	[statement]
else
	[statement]

// if else for multiple statements should be
// enclosed within "begin" and "end"

if (expression) begin
	[multiple statements]
end else begin
	[multiple statements]
end

// if-else-if statement
if (expression)
	[statement]
else if (expression)
	[statement]

else
	[statement]

```

if-else 的 else 部分是可选的，它会造成混乱。为了避免这种混淆，如果前一个没有 else，那么总是将 else 与前一个关联起来会更容易。另一种方法是将语句包含在一个开始-结束块中。最后一个 else 部分处理不满足以上条件或缺省条件的情况。

循环提供了一种在一个块中执行单个或多个语句一次或多次的方法。在 [Verilog](https://www.javatpoint.com/verilog) 中，有四种不同类型的循环语句。

### 1.永远循环

这个循环将连续执行块中的语句。

**语法**

```

Forever
      [statement]
forever begin
      [multiple statements]
end

```

**例**

```

module my_block;
initial begin
	forever begin
		$display ("This will be printed forever, simulation can hang ...");
	end
end
endmodule

```

执行上述示例后，它会生成以下数据。

```

ncsim> run
This will be printed forever, simulation can hang
This will be printed forever, simulation can hang
...
...
This will be printed forever, simulation can hang
This will be printed forever, simulation can hang 
This will be printed forever, simulation can hang 
This will be printed forever, simulation can hang 
…
…
Result reached a maximum of 5000 lines.
Killing process.

```

### 2.重复循环

这将执行固定次数的语句。如果表达式的计算结果为 X 或 Z，它将被视为零，不会被执行。

**语法**

```

repeat ([num_of_times]) begin
	[statements]
end
repeat ([num_of_times]) @ ([some_event]) begin
	[statements]
end

```

**例**

```

module my_block;
	initial begin
		repeat(5) begin
			$display("This is a new iteration ...");
		end
	end
endmodule

```

上面的代码生成了以下结果。

```

ncsim> run
This is a new iteration
This is a new iteration 
This is a new iteration 
This is a new iteration 
This is a new iteration 

ncsim: *W,RNQUIE: Simulation is complete.

```

### 第三步

只要表达式为真，它就会执行语句，一旦条件变为假，它就会退出。如果条件从一开始就是假的，语句将根本不会被执行。

**语法**

```

while (expression) begin
	[statements]
end

```

**例**

```

module my_block;
  	integer i = 5;
	initial begin
      while (i > 0) begin
        $display ("Iteration#%0d", i);
        i = i - 1;
      end
	end
endmodule

```

运行上面的代码，我们将获得以下输出。

```

ncsim> run
Iteration#5
Iteration#4
Iteration#3
Iteration#2
Iteration#1
ncsim: *W,RNQUIE: Simulation is complete.

```

### 4.For 循环

For 循环使用三步过程控制语句:

*   初始化循环计数器变量。
*   计算表达式，通常涉及循环计数器变量。
*   递增循环计数器变量，以便表达式在以后变为 false，并且循环将退出。

**语法**

```

for ( initial_assignment; condition; increment_variable) begin
	[statements]
end

```

**例**

```

module my_block;
  	integer i = 5;
	initial begin
      for (i = 0; i < 5; i = i + 1) begin
        $display ("Loop #%0d", i);
      end
    end
endmodule

```

执行 for 循环代码后，输出如下所示

```

ncsim> run
Loop #0
Loop #1
Loop #2
Loop #3
Loop #4
ncsim: *W,RNQUIE: Simulation is complete.

```

* * *