# Verilog定时控制

> 原文：<https://www.javatpoint.com/verilog-timing-control>

模拟中需要定时控制语句来提前时间。应使用时序控制来指定程序语句的执行时间。

[Verilog](https://www.javatpoint.com/verilog) 中有以下几种定时控制:

*   延迟控制
*   边缘敏感事件控制
*   级别敏感事件控制
*   命名事件

***延迟*** 控制是在模拟器遇到语句时和语句执行时之间增加延迟的一种方式。

***事件*** 表达式允许语句被延迟，直到某个模拟事件发生，该模拟事件可以改变网络或变量的值，或者在另一个过程中触发的显式命名事件。

模拟时间可以通过以下方法之一提前。网和门已经被建模为具有内部延迟，也提前了模拟时间。

### 延迟控制

延迟控制是通过指定遇到语句时执行的等待时间来实现的。符号#用于指定延迟。

我们可以通过三种方式指定基于延迟的时序控制:

1.  **常规延迟控制:**将在程序分配中指定为非零数字。
2.  **分配内延迟控制:**在这种情况下，将在分配操作员的右侧指定延迟。右侧表达式将在当前时间进行计算，并且赋值将仅在延迟后进行。
3.  **零延迟控制:**零延迟控制语句指定程序赋值左侧的零延迟值。此方法用于确保语句在模拟时间结束时执行。这意味着，零延迟控制语句在该模拟时间内的所有其他语句执行完毕后执行。

### 事件控制

事件控制语句或语句块的执行。变量和网上的值变化可以用作同步事件来触发其他过程语句的执行，并且是 ***隐式*** 事件。

事件基于像朝 0 这样的变化方向，使其成为*，朝 1 变化使其成为 ***posedge*** 。*

 **   负边缘是从 1 到 X、Z 或 0 以及从 X 或 Z 到 0 的过渡
*   偏边是从 0 到 X、Z 或 1 以及从 X 或 Z 到 1 的过渡

从同一个状态到同一个状态的转变不应该是边缘。边缘事件只能在矢量信号或变量的最低有效位上检测到。

如果表达式的计算结果相同，则不能将其视为事件。有不同类型的基于事件的控件。

**1。常规事件控制:**语句的执行将在信号变化或信号的正或负转换时发生。例如，时钟的正边沿和复位的负边沿等。

```

@(posedge clock) out = in;
@(clock) z = n << 2;

```

**2。命名事件控制:**事件关键字可用于声明可显式触发的命名事件。

事件不能包含任何数据，没有持续时间，并且可以在任何特定时间发生。

命名事件由->运算符通过在命名事件句柄前加前缀来触发。可以通过@运算符来等待命名事件。

```

module tb;
  event a_event;
  event b_event[5];

  initial begin
    #20 -> a_event;

    #30;
    ->a_event;

    #50 ->a_event;
    #10 ->b_event[3];
  end

  always @ (a_event) $display ("T=%0t [always] a_event is triggered", $time);

  initial begin
    #25;
    @(a_event) $display ("T=%0t [initial] a_event is triggered", $time);

    #10 @(b_event[3]) $display ("T=%0t [initial] b_event is triggered", $time);
  end

endmodule

```

命名事件用于同步两个或多个并发运行的进程。事件可以声明为数组。

例如， ***【始终】*** 区块和第二个 ***初始*** 区块由 a_event 同步。事件可以声明为数组，例如在 b_event 的情况下，它是一个大小为 5 的数组，索引 3 用于触发和等待目的。

**3。事件或控制:**信号或事件的转换可以触发语句的执行，如下所示。

or 运算符可以等到表达式中触发任何一个列出的事件。逗号(，)可以用来代替或运算符。

```

always @(clock or in)                         //Wait for the clock or in to change
OR
always @(clock, in)

```

### 边缘敏感事件控制

在 Verilog 中，@字符指定了一个边缘敏感的事件控件，该控件会一直阻塞，直到某个事件标识符的值发生转换(边缘)。

边缘事件被排队，然后由@(...)警卫，而不是@(？)作为等待边缘事件的守卫，阻塞直到边缘事件发生。

对@ ()来说唯一重要的边缘事件...)守护是那些发生在它等待的时候。在到达守卫之前发生的边缘事件与守卫无关。

### 级别敏感事件控制

程序语句的执行可以延迟，直到条件变为真，并使用*关键字完成。这是一个对级别敏感的控件。*

 *wait 语句计算一个条件，如果条件为假，过程语句将一直被阻塞，直到条件变为真。

```

module tb;
  reg [3:0] ctr;
  reg clk;

  initial begin
    {ctr, clk} <= 0;
    wait (ctr);
    $display ("T=%0t Counter reached non-zero value 0x%0h", $time, ctr);
    wait (ctr == 4) $display ("T=%0t Counter reached 0x%0h", $time, ctr);
    $finish;
  end
  always #10 clk = ~clk;
always @ (posedge clk)
    ctr <= ctr + 1;
endmodule

```

执行完成后，它会产生以下输出:

```

ncsim> run
T=10 Counter reached non-zero value 0x1
T=70 Counter reached 0x4
T=90 Counter reached 0x5
T=170 Counter reached 0x9
Simulation complete via $finish(1) at time 170 NS + 1

```

### 隐式事件表达式列表

事件表达式列表或灵敏度列表通常是 RTL 中许多功能错误的常见原因，因为用户在程序块中引入新信号后可能会忘记更新灵敏度列表。

```

module tb;
	reg a, b, c, d;
	reg x, y;

 // signals inside () after @ operator 
	// it is a, b, c or d

	always @ (a, b, c, d) begin
		x = a | b;
		y = c ^ d;
	end

	initial begin
             $monitor ("T=%0t a=%0b b=%0b c=%0b d=%0b x=%0b y=%0b", $time, a, b, c, d, x, y);
		{a, b, c, d} = 0;

		#10 {a, b, c, d} = $random;
		#10 {a, b, c, d} = $random;
		#10 {a, b, c, d} = $random;
	end

endmodule

```

现在执行上面的代码，我们将得到以下输出:

```
ncsim> run
T=0 a=0 b=0 c=0 d=0 x=0 y=0
T=10 a=0 b=1 c=0 d=0 x=1 y=0
T=20 a=0 b=0 c=0 d=1 x=0 y=1
T=30 a=1 b=0 c=0 d=1 x=1 y=1
ncsim: *W,RNQUIE: Simulation is complete.

```

如果用户决定添加新信号 e 并将反相信号捕获到 z 中，则必须特别注意将 e 也添加到灵敏度列表中。

```

module tb;
	reg a, b, c, d, e;
	reg x, y, z;

  // Add "e" also into sensitivity list
  always @ (a, b, c, d, e) begin
		x = a | b;
		y = c ^ d;
    	z = ~e;
	end

	initial begin
      $monitor ("T=%0t a=%0b b=%0b c=%0b d=%0b e=%0b x=%0b y=%0b z=%0b",
                				$time, a, b, c, d, e, x, y, z);
      {a, b, c, d, e} = 0;

      #10 {a, b, c, d, e} <= $random;
      #10 {a, b, c, d, e} <= $random;
      #10 {a, b, c, d, e} <= $random;
	end

endmodule

```

输出如下所示:

```
ncsim> run
T=0 a=0 b=0 c=0 d=0 e=0 x=0 y=0 z=1
T=10 a=0 b=0 c=1 d=0 e=0 x=0 y=1 z=1
T=20 a=0 b=0 c=0 d=0 e=1 x=0 y=0 z=0
T=30 a=0 b=1 c=0 d=0 e=1 x=1 y=0 z=0
ncsim: *W,RNQUIE: Simulation is complete.

```

Verilog 允许用*替换敏感度列表，这是一种方便的简写方式，通过添加语句读取的所有网络和变量来消除这些问题，如下代码所示。

```

module tb;
	reg a, b, c, d, e;
	reg x, y, z;

  // Use @* or @(*)
  always @ * begin
		x = a | b;
		y = c ^ d;
    	z = ~e;
	end

	initial begin
      $monitor ("T=%0t a=%0b b=%0b c=%0b d=%0b e=%0b x=%0b y=%0b z=%0b",
                				$time, a, b, c, d, e, x, y, z);
      {a, b, c, d, e} = 0;

      #10 {a, b, c, d, e} <= $random;
      #10 {a, b, c, d, e} <= $random;
      #10 {a, b, c, d, e} <= $random;
	end

endmodule

```

输出如下所示:

```
ncsim> run
T=0 a=0 b=0 c=0 d=0 e=0 x=0 y=0 z=1
T=10 a=0 b=0 c=1 d=0 e=0 x=0 y=1 z=1
T=20 a=0 b=0 c=0 d=0 e=1 x=0 y=0 z=0
T=30 a=0 b=1 c=0 d=0 e=1 x=1 y=0 z=0
ncsim: *W,RNQUIE: Simulation is complete.

```

* * ***