# 门级建模

> 原文：<https://www.javatpoint.com/verilog-gate-level-modeling>

在 Verilog 中，大多数数字设计都是在更高的抽象层次上完成的，比如 RTL。然而，通过使用诸如*和 ***或*** 的组合元件，在较低的级别构建较小的确定性电路变得很自然。*

 *在这一级完成的建模被称为门级建模，因为它涉及*门，并且在硬件原理图和 Verilog 代码之间具有一对一的关系。*

 *[Verilog](https://www.javatpoint.com/verilog) 支持一些被称为 ***原语*** 的基本逻辑门，因为它们可以被实例化，比如模块，并且它们已经被预定义了。

门级建模实际上是最底层的抽象，因为很少使用开关级抽象。门级建模用于实现设计中最底层的模块，如 ***多路复用器、全加器*** 等。Verilog 有所有基本门的门原语。

Verilog 支持内置的原始门建模。支持的门有 ***、多输入多输出、三态*** 、 ***拉门*** 。

多输入门为 ***、与非、或、或非、异或、*** 和***【Xnor】***，其输入数为两个或更多，且只有一个输出。

多输出门是*和 ***而不是*** ，它们的输出是一个或多个，只有一个输入。*

 *该语言还支持三态门的建模，包括***buff 0、bufif1、notif0、*** 和 ***notif1*** 。这些门有一个输入、一个控制信号和一个输出。

拉门为 ***上拉*** 和 ***下拉*** ，只有一个输出。

**语法**

以下是每种零延迟门的基本语法，例如:

```

and | nand | or | nor | xor | xnor [instance name] (out, in1, ..., inN);      // [] is optional and | is selection 
buf | not [instance name] (out1, out2, ..., out2, input); 
bufif0 | bufif1 | notif0 | notif1 [instance name] (outputA, inputB, controlC); 
pullup | pulldown [instance name] (output A); 

```

在一个构造中，同一类型的门也可以有多个实例，用逗号分隔:

```

and [inst1] (out11, in11, in12), [inst2] (out21, in21, in22, in23), [inst3] (out31, in31, in32, in33); 

```

当电路是简单的组合时，门级建模是有用的，例如 ***多路复用器*** 。多路复用器是一个简单的电路，它将许多输入中的一个连接到一个输出。

### 门基元

门原语是 Verilog 中预定义的模块，可以随时使用。有两类门基元:

1.  单输入门基元
2.  多输入门基元

### 1.单输入门基元

单输入门基元具有单个输入和一个或多个输出。门基元有 ***not、buf、notif*** ，而***buff***也有控制信号。

仅当控制信号有效时，栅极才会传播，否则输出为高阻抗状态。

**1.1 不是，buf Gates**

这些门只有一个标量输入，但可能有多个输出。

**buf** 代表一个缓冲器，在极性没有任何变化的情况下，将值从输入传输到输出。

**而不是**代表将输入端信号极性反转的反相器。所以输入端的 0 将产生 1，反之亦然。

**语法**

```

module gates (	input a, b,
				output c, d);

	buf (c, a, b); 	// c is the output, a and b are inputs
  	not (d, a, b);	// d is the output, a and b are inputs

endmodule

```

**例**

```

module tb;
   reg a, b;
   wire c, d;
   integer i;
gates u0 ( .a(a), .b(b), .c(c), .d(d));
   initial begin
	{a, b} = 0;
   $monitor ("[T=%0t a=%0b b=%0b c(buf)=%0b d(not)=%0b", $time, a, b, c, d);
	for (i = 0; i < 10; i = i+1) begin
		#1 	a <= $random;
			b <= $random;
	end
end
endmodule

```

1.2 bufif，notf gates

Bufif 和 notif 基元分别是缓冲器和反相器，通过 bufif 和 notif 基元可以获得使能输出的附加控制信号。

这些门只有在控制信号使能时才具有有效输出，否则输出将为高阻抗。

有两种版本，一种是用 1 表示的正常控制极性，如 bufif1 和 notif1。其次，控制的反极性由 0 表示，如 bufif0 和 notif0。

**语法**

```

module bufif_notif_gates (output c, d, input a, b);
bufif (c, a, b);   // c is the output, a and b are inputs
notif (d, a, b);  // d is the output, a and b are inputs
endmodule

```

**例**

```

module bufif_notif_gates_tb;
  reg a, b;
  wire c, d;
  bufif_notif_gates Instance0 (c, d, a, b);
  initial begin
    a = 0; b = 0;
 #1 a = 0; b = 1; 
 #1 a = 1; b = 0;
 #1 a = 1; b = 1;
  end
  initial begin
    $monitor ("T=%t| a=%b |b=%b| c(bufif)=%b |d(notif)=%b", $time, a, b, c, d);
  end
endmodule

```

### 2.多输入门基元

多个输入门原语包括 ***、【或】、【异或】、【与非门】、*、**、*和【XNOR】*、**。它们可能有多个输入和一个输出。**

**2.1 与门、或门、异或门**

“与”、“或”和“异或门”需要多个标量输入，并产生单个标量输出。

这些原语的参数列表中的第一个终端是输出，它随着任何输入的移动而改变。

**语法**

```

module and_or_xor_gates (output c, d, e, input a, b);
  and (c, a, b);   // c is the output, a and b are inputs
  or  (d, a, b);  // d is the output, a and b are inputs
  xor (e, a, b);   // e is the output, a and b are inputs
endmodule

```

**例**

```

module and_or_xor_gates_tb;
reg a, b;
wire c, d, e;
and_or_xor_gates Instance0 (c, d, e, a, b);
 initial begin
   a = 0; b = 0;
   #1 a = 0; b = 1; 
   #1 a = 1; b = 0;
   #1 a = 1; b = 1;
 end
 initial begin
   $monitor ("T=%t |a=%b |b=%b |c(and)=%b |d(or)=%b |e(xor)=%b", $time, a, b, c, d, e);
 end
endmodule

```

**1.2 与非门、或非门、XNOR 门**

上述所有门的逆是与非门、或非门和 XNOR 门。上面相同的设计被重复使用，只是基元与它们的反向版本互换了。

**语法**

```

module nand_nor_xnor_gates (output c, d, e, input a, b);
 nand (c, a, b); // c is the output, a and b are inputs 
 nor (d, a, b); // d is the output, a and b are inputs 
 xnor (e, a, b); // e is the output, a and b are inputs
endmodule 

```

**例**

```

module nand_nor_xnor_gates_tb;
reg a, b;
wire c, d, e;
nand_nor_xnor_gates Instance0 (c, d, e, a, b);
  initial begin
    a = 0; b = 0;
 #1 a = 0; b = 1; 
 #1 a = 1; b = 0;
 #1 a = 1; b = 1;
end
  initial begin
    $monitor ("T=%t |a=%b |b=%b |c(nand)=%b |d(nor)=%b |e(xnor)=%b", $time, a, b, c, d, e);
  end
endmodule

```

所有这些门也可以有两个以上的输入。

```

module all_gates (output x1, y1, z1, x2, y2, z2 , input a, b, c, d);
  and (x1, a, b, c, d);   // x1 is the output, a, b, c, d are inputs
  or  (y1, a, b, c, d);  // y1 is the output, a, b, c, d are inputs
  xor (z1, a, b, c, d);   // z1 is the output, a, b, c, d are inputs
  nand (x2, a, b, c, d); // x2 is the output, a, b, c, d are inputs
  nor (y2, a, b, c, d); // y2 is the output, a, b, c, d are inputs 
  xnor (z2, a, b, c, d); // z2 is the output, a, b, c, d are inputs
endmodule

```

### 多路复用器的门级建模

4x1 多路复用器的门级电路图如下所示。它用于为 4x1 多路复用器编写模块。

![Gate Level Modeling](img/1342fc16d2893c79c7738c03177bb6dd.png)

```

module 4x1_mux (out, in0, in1, in2, in3, s0, s1);

// port declarations
output out; // Output port.
input in0, in1, in2\. in3; // Input ports.
input s0, s1; // Input ports: select lines.

// intermediate wires
wire inv0, inv1; // Inverter outputs.
wire a0, a1, a2, a3; // AND gates outputs.

// Inverters.
not not_0 (inv0, s0);
not not_1 (inv1, s1);

// 3-input AND gates.
and and_0 (a0, in0, inv0, inv1);
and and_1 (a1, in1, inv0, s1);
and and_2 (a2, in2, s0, inv1);
and and_3 (a3, in3, s0, s1);

// 4-input OR gate.
or or_0 (out, a0, a1, a2, a3);

endmodule

```

### 全加器的门级建模

下面是使用半加法器的全加器的实现。

**1。半加法器**

![Gate Level Modeling](img/70fa4748d2ea0f4706ab0bc4ba38a685.png)

```

module half_adder (sum, carry, in0, in1);

output sum, carry;
input in0, in1;

// 2-input XOR gate.
xor xor_1 (sum, in0, in1);

// 2-input AND gate.
and and_1 (carry, in0, in1);

endmodule

```

**2。全加器**

![Gate Level Modeling](img/eedd6a2b0af52e38f52fc3cde2fd5a78.png)

```

module full_adder (sum, c_out, ino, in1, c_in);

output sum, c_out;
input in0, in1, c_in;

wire s0, c0, c1;
// Half adder: port connecting by order.
half_adder ha_0 (s0, c0, in0, in1);

// Half adder : port connecting by name.
half_adder ha_1 (.sum(sum),
                .in0(s0),
                .in1(c_in),
                .carry(c1));

// 2-input XOR gate, to get c_out.
xor xor_1 (c_out, c0, c1);

endmodule

```

* * ****