# Verilog 调度语义

> 原文：<https://www.javatpoint.com/verilog-scheduling-semantics>

Verilog 设计和测试平台通常有许多代码行，包括 ***【始终】*** 或 ***【初始】*** 块、连续赋值和在模拟过程中不同时间变为活动的其他程序语句。

[Verilog](https://www.javatpoint.com/verilog) 模型中信号值的每一次变化都被视为 ***更新事件*** 。并且诸如总是和分配对这些更新事件敏感的块之类的过程以任意顺序被评估，并且被称为 ***评估*事件**。

由于这些事件可以在不同的时间发生，因此通过将它们调度到按模拟时间排列的*事件队列中，可以更好地管理它们，并确保它们的正确执行顺序。*

 *```

module tb;
	reg a, b, c;
	wire d;

	// 'always' is a process that gets evaluated when either 'a' or 'b' is updated.
	// When 'a' or 'b' changes in value it is called an 'update event'. 
	// When 'always' block is triggered because of a change in 'a' or 'b' it is called an evaluation event.

	always @ (a or b) begin
		c = a & b;
	end

 // Here 'assign' is a process that is evaluated when either 'a' or 'b' or 'c' gets updated.

             assign d = a | b ^ c;
endmodule

```

### 事件队列

模拟步骤可以分成四个不同的区域。活动事件队列只是需要在当前时间执行的一组进程，导致更多的进程被调度到活动或其他事件队列中。事件可以添加到任何区域，但总是从 ***活动的*** 区域中移除。

*   ***激活*** 事件发生在当前模拟时间，可以任意顺序处理。
*   *非活动事件在当前模拟时间发生，但在所有活动事件处理完毕后被处理。*
**   ***【非阻塞分配】*** 先前评估的事件将在处理完所有活动和非活动事件后分配。*   *事件在所有活动、非活动和非阻塞任务完成后处理。**

 **当前时间步长的活动队列中的所有事件都已执行时。模拟器将时间推进到下一个时间步骤，并执行其活动队列。

```

module tb;
	reg x, y, z

	initial begin
		#1 	x = 1;
			y = 1;
		#1 	z = 0;
	end
endmodule

```

模拟从时间 0 开始，第一条语句计划在模拟时间达到 1 时间单位时执行，在该时间单位，它将 x 和 y 指定为 1。

这是当前时间的活动队列，是 1 时间单位。模拟器然后在 z 被指定为 0 的 1 个时间单位之后调度下一个语句。

### 不确定的

在模拟过程中，可能会出现竞争条件，最终导致同一设计和测试平台产生不同的输出。非确定性行为的原因之一是因为 ***活动的*** 事件可以从队列中移除，并以任何顺序进行处理。

当同时触发多个进程时，电气和电子工程师协会( *IEEE* )标准没有规定进程的执行顺序。它是任意的，并且它因模拟器而异。这就是所谓的*非决定论*。

有两种常见的非决定论案例，它们由相同的根本原因引起，但以不同的方式表现出来。

**情况 1:** 当多个语句在零时间内执行时，那么它们的执行顺序会影响结果。因此，不同的执行顺序会产生不同但正确的结果。零时间执行意味着在不提前模拟时间的情况下评估语句。

```

always @(d)
   q = d;
assign q = ~d;

```

当变量 d 改变时，这两个过程，一个程序块和一个连续赋值被安排在同一时间执行。

如果总是块首先被评估，变量 q 被总是块赋予新的 d 值。然后执行连续分配。它将 d 的新值的补码赋给变量 q。

如果首先计算连续赋值，q 得到 d 的新值的补码，然后程序赋值将新值(非补码)赋给 q，因此，两个顺序产生相反的结果。

**情况 2:** 当块中的交错过程语句同时执行时，考虑这种情况。当同时调度两个过程块时，不能保证一个块中的所有语句在另一个块中的语句开始之前完成。这两个块中的语句可以按交错顺序执行。

```

always @(posedge clk) // always block1
begin
   x = 1'b0;
   y = x;
end

always @(posedge clk) // always block2
begin
   x = 1'b1;
end

```

当 clk 的正边沿到达时，两个模块总是被触发。一种交错顺序是

```

x = 1'b0;
y = x;
x = 1'b1;

```

在这种情况下，y 得到 0。另一种交错顺序是

```

x = 1'b0;
x = 1'b1;
y = x;

```

在这种情况下，y 得到 1。

* * ****