# Verilog D 锁存器

> 原文：<https://www.javatpoint.com/verilog-d-latch>

触发器在时钟的负沿或正沿捕捉输入端的数据。重要的是，无论数据在时钟沿之后直到下一个时钟沿发生什么，都不会反映在输出中。

一 ***闩锁*** 不捕捉时钟边缘；相反，只要被断言，输出就跟随输入。

D 锁存器用于存储一位数据。D 锁存器本质上是门控 SR 锁存器的修改。

下图显示了 [Verilog](https://www.javatpoint.com/verilog) 中 D 锁存器的参数。输入 D 是要存储的数据。输入 G 用于控制存储。输出 Q 和 Qn 分别是存储数据和存储数据的补码。

![Verilog D Latch](img/6a71f19e524ef7a2ca868f35ce92a0c0.png)

### 例子

在这个例子中，我们有一个三输入一输出的锁存器。输入***【d】***代表数据，可以是 0 也可以是 1， ***rstn*** 代表低电平有效复位， ***en*** 代表使能，使输入数据锁存到输出。

![Verilog D Latch](img/c271a8e36dbe3deef0519254b662f726.png)

复位有效-低电平意味着当该输入变为 0 时，设计元件将被复位，或者当其值为低电平时，复位有效。输出 ***q*** 的值由输入 ***d、en*** 和 ***rstn*** 决定。

```

module d_latch (  input d,           // 1-bit input pin for data
                  input en,          // 1-bit input pin for enabling the latch
                  input rstn,        // 1-bit input pin for active-low reset
                  output reg q);     // 1-bit output pin for data output

   // This always block is "always" triggered whenever en/rstn/d changes
   // If reset is asserted, then the output will be zero 
   // Else as long as enable is high, output q follows input d

   always @ (en or rstn or d)
      if (!rstn)
         q <= 0;
      else
         if (en)
            q <= d;
endmodule

```

#### 注:始终块的灵敏度列表包含更新输出所需的所有信号。

只要灵敏度列表中的任何信号改变其值，就会触发该模块。同样， ***q*** 只有在 ***en*** 为高电平时才会得到 d 的值，因此是一个 ***正*** 锁存器。

### 硬件示意图

![Verilog D Latch](img/e9357f4d128ed48eefe113663514a6ea.png)

### 试验台

```

module tb_latch;
   // Declare variables that can be used to drive values to the design
   reg d;
   reg en;
   reg rstn;
   reg [2:0] delay;
   reg [1:0] delay2;
   integer i;

   // Instantiate design and connect design ports with TB signals
   d_latch  dl0 ( .d (d),
                  .en (en),
                  .rstn (rstn),
                  .q (q));

   // This initial block forms the stimulus to test the design
   initial begin
      $monitor ("[%0t] en=%0b d=%0b q=%0b", $time, en, d, q);

      // 1\. Initialize testbench variables
      d <= 0;
      en <= 0;
      rstn <= 0;

      // 2\. Release reset
      #10 rstn <= 1;

      // 3\. Randomly change d and enable
      for (i = 0; i < 5; i=i+1) begin
         delay = $random;
         delay2 = $random;
         #(delay2) en <= ~en;
         #(delay) d <= i;
      end
   end
endmodule

```

为了使我们的测试平台以更随机的方式断言和取消断言信号，我们声明了一个名为 ***延迟*** 的寄存器变量，大小为 3 位，因此它可以取 0 到 7 之间的任何值。然后 ***延迟*** 变量用于延迟 ***d*** 和 ***en*** 的赋值，以在每个循环中获得不同的模式。

上述代码产生以下输出，例如:

```
ncsim> run
[0] en=0 d=0 q=0
[11] en=1 d=0 q=0
[18] en=0 d=0 q=0
[19] en=0 d=1 q=0
[20] en=1 d=1 q=1
[25] en=1 d=0 q=0
[27] en=0 d=0 q=0
[32] en=0 d=1 q=0
[33] en=1 d=1 q=1
[34] en=1 d=0 q=0
ncsim: *W,RNQUIE: Simulation is complete.

```

* * *