# Verilog 模数计数器

> 原文：<https://www.javatpoint.com/verilog-mod-n-counter>

计数器是时序逻辑器件，遵循由外部时钟(CLK)信号触发的预定计数状态序列。特定计数器在返回其原始第一状态之前前进的状态数或计数序列称为 ***模数*** (MOD)。换句话说，模数(或模)是计数器计数的状态数，是计数器的除数。

***【模数】*** 计数器，或 ***MOD*** 计数器，是根据计数器在返回初始值之前将排序的状态数来定义的。

例如，从二进制的 00 <sub>2</sub> 到 11 <sub>2</sub> 计数的 2 位计数器，十进制的 0 到 3，模数值为 4 ( 00 → 1 → 10 → 11，返回 00)；因此被称为模 4 或模 4 计数器。还要注意，从 00 到 11 需要四个时钟脉冲。

在本例中，只有两位(n = 2)，那么计数器可能输出状态的最大数量(最大模数)为 2 <sup>n</sup> = 2 <sup>2</sup> 或 4。然而，计数器可以被设计成通过将多个计数级级联在一起以产生单个模数或模数计数器来计数到它们序列中的任意 2 个 T4 状态。

因此，“Mod-N”计数器将需要连接“N”个触发器来计数单个数据位，同时提供 2 <sup>n</sup> 种不同的输出状态(N 是位数)。请注意，N 始终是一个整数值。

然后我们可以看到，MOD 计数器的模数值是 2 的整数次幂，即 2、4、8、16 等，根据所用触发器的数量和它们的连接方式，产生一个 n 位计数器，从而确定计数器的类型和模值。

### 例子

```

module modN_ctr
  # (parameter N = 10,
     parameter WIDTH = 4)

  ( input   clk,
    input   rstn,
   	output  reg[WIDTH-1:0] out);

  always @ (posedge clk) begin
    if (!rstn) begin
      out <= 0;
    end else begin
      if (out == N-1)
        out <= 0;
      else
        out <= out + 1;
    end
  end
endmodule

```

**试验台**

下面是上面给出的相同示例的测试平台，例如:

```

module tb;
  parameter N = 10;
  parameter WIDTH = 4;

  reg clk;
  reg rstn;
  wire [WIDTH-1:0] out;

  modN_ctr u0  ( 	.clk(clk),
                	.rstn(rstn),
                	.out(out));

  always #10 clk = ~clk;

  initial begin
    {clk, rstn} <= 0;

    $monitor ("T=%0t rstn=%0b out=0x%0h", $time, rstn, out);
    repeat(2) @ (posedge clk);
    rstn <= 1;

    repeat(20) @ (posedge clk);
    $finish;
  end
endmodule

```

输出如下所示:

```
ncsim> run
T=0 rstn=0 out=0xx
T=10 rstn=0 out=0x0
T=30 rstn=1 out=0x0
T=50 rstn=1 out=0x1
T=70 rstn=1 out=0x2
T=90 rstn=1 out=0x3
T=110 rstn=1 out=0x4
T=130 rstn=1 out=0x5
T=150 rstn=1 out=0x6
T=170 rstn=1 out=0x7
T=190 rstn=1 out=0x8
T=210 rstn=1 out=0x9
T=230 rstn=1 out=0xa
T=250 rstn=1 out=0x0
T=270 rstn=1 out=0x1
T=290 rstn=1 out=0x2
T=310 rstn=1 out=0x3
T=330 rstn=1 out=0x4
T=350 rstn=1 out=0x5
T=370 rstn=1 out=0x6
T=390 rstn=1 out=0x7
T=410 rstn=1 out=0x8
Simulation complete via $finish(1) at time 430 NS + 0

```

### Mod 6 上行计数器

这是一个计数器，在选定的数字复位。例如，一个两位数的十进制计数器，留给它自己的设备，将从 00 计数到 99。除非你有 100 秒钟，否则这对于一个时钟来说没有多大用处。

![Verilog Mod-N Counter](img/fce38cbdb6378e6d98bcccefca5304aa.png)

要解决这个问题，计数器必须从 00 转到 59。这是通过检测左边数字中的 6 并使用它将计数器重置为零来实现的。这将是一个模 6 计数器或 60，如果我们包括两个数字。

![Verilog Mod-N Counter](img/4ee84a94a2259c6256f0e4659b2f029d.png)

### Mod 5 向下计数器

假设我们想设计一个 MOD-5 计数器。首先我们知道“m = 5”，所以 2 <sup>n</sup> 必须大于 5。

![Verilog Mod-N Counter](img/5c94501056a025d6b2b562ac278cf2e1.png)

由于 2 <sup>1</sup> = 2，2 <sup>2</sup> = 4，2 <sup>3</sup> = 8，并且 8 大于 5，那么我们需要一个带有三个触发器(N = 3)的计数器，给我们一个二进制(0 到 7 十进制)的 000 到 111 的自然计数。

![Verilog Mod-N Counter](img/5821b1397c294571b5739b05632afe91.png)

* * *