# Verilog 约翰逊计数器

> 原文：<https://www.javatpoint.com/verilog-johnson-counter>

约翰逊计数器是一种数字电路，由一系列以反馈方式连接的触发器组成。Verilog Johnson 计数器是一个计数器，如果位数为 n，则计数 2N 个状态

该电路是一种特殊类型的移位寄存器，其中最后一个触发器的补码输出反馈到第一个触发器的输入。这几乎类似于环形计数器，有一些额外的优势。

约翰逊计数器的主要优点是，与标准环形计数器相比，它只需要一半的触发器数量，然后它的模数减半。因此一个 ***n 级*** 约翰逊计数器将循环一个数据位，给出 2n 个不同状态的序列，因此可以认为是一个 ***mod-2n*** 计数器。

最后一个触发器的反相输出 Q 连接回第一个触发器的输入 D。下面是 4 位约翰逊计数器的电路图:

![Verilog Johnson Counter](img/a5e04e02af3fb87d8d653b712ad0b498.png)

Q 在反馈到输入 D 之前的这种反转导致计数器以不同的方式对*进行计数。而不是像普通的环形计数器那样通过一组固定的模式进行计数，例如对于 4 位计数器，“0001”(1)、“0010”(2)、“0100”(4)、“1000”(8)并重复。当初始逻辑“1”向右通过时，约翰逊计数器先递增后递减，取代前面的逻辑“0”。*

 *4 位约翰逊环计数器传递四个逻辑“0”块，然后传递四个逻辑“1”，从而产生 8 位模式。

当反相输出 Q 连接到输入 D 时，这个 8 位模式不断重复。例如“1000”、“1100”、“1110”、“1111”、“0111”、“0011”、“0001”、“0000”。下表说明了这一点:

| 时钟脉冲号 | 国外货运代理人(Foreign Freight Agent) | 纤维支气管镜 | 固定性屈曲挛缩 | 焦点胶片距离 |
| Zero | Zero | Zero | Zero | Zero |
| one | one | Zero | Zero | Zero |
| Two | one | one | Zero | Zero |
| three | one | one | one | Zero |
| four | one | one | one | one |
| five | Zero | one | one | one |
| six | Zero | Zero | one | one |
| seven | Zero | Zero | Zero | one |

除了围绕连续循环对数据进行计数或旋转之外，环形计数器还可用于检测或识别一组数据中的各种模式或数值。通过将简单的逻辑门如 ***或*** 门连接到触发器的输出端，可以使电路检测设定的数量或值。

标准的 2 级、3 级或 4 级**约翰逊环计数器**也可以通过改变其反馈连接来划分时钟信号频率，并且还提供 3 分频或 5 分频输出。

例如，通过连接到 A、B 和 NOT-B 的数据输出，三级约翰逊环计数器可以用作三相、120 度相移方波发生器

标准的 5 级约翰逊计数器，如常见的 CD4017，通常用作同步十进制计数器/除法器电路。

其他组合，如较小的 2 级电路，也称为*振荡器或发生器，可用于产生四个独立的输出，每个输出彼此相差 90 度，以产生 4 相定时信号。*

 *### 例子

```

module johnson_ctr #(parameter WIDTH=4)
  (
	input clk,
	input rstn,
  	output reg [WIDTH-1:0] out
  );

  always @ (posedge clk) begin
      if (!rstn)
         out <= 1;
      else begin
        out[WIDTH-1] <= ~out[0];
        for (int i = 0; i < WIDTH-1; i=i+1) begin
          out[i] <= out[i+1];
        end
      end
  end
endmodule

```

**试验台**

```

module tb;
  parameter WIDTH = 4;

  reg clk;
  reg rstn;
  wire [WIDTH-1:0] out;

  johnson_ctr 	u0 (.clk (clk),
                .rstn (rstn),
                .out (out));

  always #10 clk = ~clk;

  initial begin
    {clk, rstn} <= 0;

    $monitor ("T=%0t out=%b", $time, out);
    repeat (2) @(posedge clk);
    rstn <= 1;
    repeat (15) @(posedge clk);
    $finish;
  end
endmodule

```

输出如下所示:

```
ncsim> run
T=0 out=xxxx
T=10 out=0001
T=50 out=0000
T=70 out=1000
T=90 out=1100
T=110 out=1110
T=130 out=1111
T=150 out=0111
T=170 out=0011
T=190 out=0001
T=210 out=0000
T=230 out=1000
T=250 out=1100
T=270 out=1110
T=290 out=1111
T=310 out=0111
Simulation complete via $finish(1) at time 330 NS + 0

```

* * ***