# 双向移位寄存器

> 原文：<https://www.javatpoint.com/verilog-bidirectional-shift-register>

触发器可以用来存储一位二进制数据(1 或 0)。然而，为了存储多位数据，我们需要多个触发器。n 个触发器将被连接以存储 n 位数据。

A ***寄存器*** 是用于存储此类信息的设备。它是一组串联的触发器，用于存储多位数据。

存储在这些寄存器中的信息可以使用 ***移位寄存器*** 进行传输。

一个 ***移位寄存器*** 是触发器的级联，其中一个触发器的输出引脚 ***q*** 连接到下一个数据输入引脚(d)。因为所有触发器都工作在同一个时钟上，所以存储在移位寄存器中的位阵列将移位一个位置。

例如，如果一个 5 位右移位寄存器的初始值为 10110，并且移位寄存器的输入连接到 0，那么下一个模式将是 01011 和下一个 00101。

![Bidirectional Shift Register](img/e1f8014b24a07aa249633cc8484bf32d.png)

***【双向移位寄存器】*** 是能够根据所选模式左右移位数据的存储设备。

下图显示了一个具有串行数据加载和检索能力的 n 位双向移位寄存器。最初，寄存器中的所有触发器都通过将其清零引脚驱动为高电平来复位。

![Bidirectional Shift Register](img/45939c906f553361ea05cb51c020ab4d.png)

***R/ L*** 控制线变低或变高，以分别选择数据位的左移或右移。

n 位移位寄存器可以通过连接 n 个触发器形成，其中每个触发器存储一位数据。

将位左移的寄存器称为 ***左移寄存器*** 。向右移位的寄存器称为 ***向右移位寄存器*** 。移位寄存器基本上有四种类型，例如:

1.  串行输入串行输出移位寄存器
2.  串行输入并行输出移位寄存器
3.  并行输入串行输出移位寄存器
4.  并行输入并行输出移位寄存器

### 例子

在这个移位寄存器示例中，我们采用五个输入和一个 n 位输出，并且使用参数 MSB 来表示移位寄存器的宽度，从而将设计参数化。

如果 n 为 4，则它成为 4 位移位寄存器。如果 n 是 8，则它成为一个 8 位移位寄存器。该移位寄存器有几个关键特性:

*   可以通过驱动 ***en*** 引脚来启用或禁用。
*   当 ***方向*** 被驱动时，它既可以向左移动，也可以向右移动。
*   如果***【rstn】***拉低，将复位移位寄存器，输出变为 0。
*   移位寄存器的输入数据值可由***【d】***引脚控制。

```

module shift_reg  #(parameter MSB=8) (  input d,               // Declare input for data to the first flop in the shift register
                                        input clk,                    // Declare input for the clock to all flops in the shift register 
                                        input en,                     // Declare input for enable to switch the shift register on/off
                                        input dir,                    // Declare input to shift in either left or right direction
                                        input rstn,                   // Declare input to reset the register to a default value
                                        output reg [MSB-1:0] out);    // Declare output to read out the current value of all flops in this register

   // This always block will "always" be triggered on the rising edge of the clock 
   // Once it enters the block, it will first check to see if reset is 0 and if yes, then reset register 
   // If no, then check to see if the shift register is enabled
   // If no => maintain previous output. If yes, then shift based on the requested direction

   always @ (posedge clk)
      if (!rstn)
         out <= 0;
      else begin
         if (en)
            case (dir)
               0 :  out <= {out[MSB-2:0], d};
               1 :  out <= {d, out[MSB-1:1]};
            endcase
         else
            out <= out;
      end
endmodule

```

**硬件示意图**

![Bidirectional Shift Register](img/52a8ab65f83ba40a3ae32f41836eef7b.png)

**试验台**

测试台用于验证该移位寄存器的功能。该示例被实例化到顶部的 ***模块*** 中，输入由不同的值驱动。每个输入的行为可以在输出 ***out*** 引脚上观察到。

![Bidirectional Shift Register](img/48ec7c0e1043c29724e1057c059f1b53.png)

```

module tb_sr;
   parameter MSB = 16;        // [Optional] Declare a parameter to represent number of bits in shift register

   reg data;                  // Declare a variable to drive d-input of design
   reg clk;                   // Declare a variable to drive clock to the design
   reg en;                    // Declare a variable to drive enable to the design
   reg dir;                   // Declare a variable to drive direction of shift register
   reg rstn;                  // Declare a variable to drive reset to the design
   wire [MSB-1:0] out;        // Declare a wire to capture output from the design

   // Instantiate design (16-bit shift register) by passing MSB and connect with TB signals
   shift_reg  #(MSB) sr0  (  .d (data),
                             .clk (clk),
                             .en (en),
                             .dir (dir),
                             .rstn (rstn),
                             .out (out));

   // Generate clock time period = 20ns, freq => 50MHz
   always #10 clk = ~clk;

   // Initialize variables to default values at time 0
   initial begin
      clk <= 0;
      en <= 0;
      dir <= 0;
      rstn <= 0;
      data <= 'h1;
   end

   // Drive main stimulus to the design to verify if this works
   initial begin

      // 1\. Apply reset and deassert reset after some time
      rstn <= 0;
      #20 rstn <= 1;
          en <= 1;

	  // 2\. For 7 clocks, drive alternate values to data pin
      repeat (7) @ (posedge clk)
         data <= ~data;

     // 3\. Shift direction and drive alternate value to data pin for another 7 clocks
      #10 dir <= 1;
      repeat (7) @ (posedge clk)
         data <= ~data;

      // 4\. Drive nothing for the next 7 clocks, allow shift register to shift based on dir simply
      repeat (7) @ (posedge clk);

      // 5\. Finish the simulation
      $finish;
   end

   // Monitor values of these variables and print them into the log file for debug
   Initial
      $monitor ("rstn=%0b data=%b, en=%0b, dir=%0b, out=%b", rstn, data, en, dir, out);
endmodule

```

### 串行输入串行输出移位寄存器(SISO)

移位寄存器允许串行输入(通过一条数据线一位接一位)并产生串行输出，称为串行输入串行输出移位寄存器。

由于只有一个输出，数据以串行模式同时离开移位寄存器一位，因此称为串行输入串行输出移位寄存器。

下面给出的逻辑电路显示了一个串行输入串行输出移位寄存器。该电路由四个串联的 D 触发器组成。

所有这些触发器都是同步的，因为相同的时钟信号施加在每个触发器上。

![Bidirectional Shift Register](img/3c94119e7142774e260a1d3631dbe40d.png)

上述电路是右移位寄存器的一个例子，从触发器的左侧获取串行数据输入。SISO 的主要用途是作为延迟元件。

### 串行输入并行输出移位寄存器

移位寄存器允许串行输入(通过一条数据线一位接一位)并产生并行输出，称为串行输入并行输出移位寄存器。

下面给出的逻辑电路显示了一个串行-并行-输出移位寄存器。该电路由四个相连的 [D 触发器](https://www.javatpoint.com/verilog-d-flip-flop)组成。

清零(CLR)信号被连接，时钟信号被连接到所有 4 个触发器以复位它们。第一个触发器的输出连接到下一个触发器的输入，以此类推。

所有这些触发器都是同步的，因为相同的时钟信号施加在每个触发器上。

![Bidirectional Shift Register](img/fec8e8a7a7b111e07ba291328e0254c5.png)

上述电路是右移位寄存器的一个例子，从触发器的左侧获取串行数据输入，并产生并行输出。

它们用于通信线路中，因为 SIPO 寄存器的主要用途是将串行数据转换为并行数据，所以需要将一条数据线解复用为多条并行线。

### 并行输入串行输出移位寄存器(PISO)

移位寄存器允许并行输入(数据分别同时提供给每个触发器)并产生串行输出，称为并行输入串行输出移位寄存器。

下面给出的逻辑电路显示了一个并行-串行-输出移位寄存器。该电路由四个相连的 D 触发器组成。

时钟输入直接连接到所有触发器。输入数据仍然通过每个触发器输入端的多路复用器单独连接到每个触发器。

前一个触发器和并行数据输入的输出连接到多路复用器输入，多路复用器的输出连接到下一个触发器。

所有这些触发器都是同步的，因为相同的时钟信号被施加到每个触发器。

![Bidirectional Shift Register](img/cbccff78758507e093987e4a8fbf059c.png)

一种并行输入串行输出(PISO)移位寄存器，用于将并行数据转换为串行数据。

### 并行输入并行输出移位寄存器(PIPO)

移位寄存器允许并行输入(数据分别同时提供给每个触发器)，并产生并行输出，称为并行输入并行输出移位寄存器。

下面给出的逻辑电路显示了一个并行输入并行输出移位寄存器。该电路由四个相连的 D 触发器组成。清除(CLR)信号和时钟信号连接到所有 4 个触发器。

在这种类型的寄存器中，各个触发器之间没有互连，因为不需要数据的串行移位。

数据以同样的方式作为输入分别提供给每个触发器，输出也分别从每个触发器收集。

![Bidirectional Shift Register](img/2b6eec4fb0750af0ed3d349abcf7fd34.png)

并行输入并行输出(PIPO)移位寄存器被用作临时存储设备，并且像 SISO 移位寄存器一样，它充当延迟元件。

### 双向移位寄存器

如果我们把一个二进制数向左移动一个位置，就相当于把这个数乘以 2。如果我们把一个二进制数向右移动一个位置，就相当于把这个数除以 2。执行这些操作；我们需要一个可以双向移位数据的寄存器。

***【双向移位】*** 寄存器是能够根据所选模式向右或向左移位数据的寄存器。

如果选择的模式是 1(高)，则数据将向右移动，如果选择的模式是 0(低)，则数据将向左移动。

下面给出的逻辑电路显示了一个双向移位寄存器。该电路由四个相连的 D 触发器组成。

输入数据连接在电路的两端，根据选择的模式，只有一个与门处于活动状态。

![Bidirectional Shift Register](img/90054afd93cf523464ec91c43ef58d37.png)

### 移位寄存器的应用

以下是移位寄存器的应用，例如:

*   移位寄存器用于临时数据存储。
*   移位寄存器也用于数据传输和数据处理。
*   串行输入串行输出和并行输入并行输出移位寄存器用于产生数字电路的时间延迟。
*   串行输入并行输出移位寄存器用于将串行数据转换为并行数据。它们用于通信线路中，其中需要将一条数据线解复用为几条平行线。
*   并行输入串行输出移位寄存器用于将并行数据转换为串行数据。

* * *