# Verilog 全加器

> 原文：<https://www.javatpoint.com/verilog-full-adder>

全加器是一个数字部件，它执行三个数字，并使用逻辑门实现。它是处理器算术逻辑单元内部的主要组件，用于增加地址、表索引、缓冲区指针和其他需要加法的地方。

一位全加器将三个一位二进制数、两个输入位、一个进位位相加，并输出一个和和一个进位位。

全加器是由两个半加法器和 ***或*** 的最终输出组成的。半加法器将两个二进制数相加。全加器是一个组合电路，因此可以用 [Verilog 语言](https://www.javatpoint.com/verilog)建模。

两个输出*和 ***进位*** 的逻辑表达式如下。a、B 是两位二进制数的输入变量，Cin 是进位输入，Cout 是 Sum 和 carry 的输出变量。*

*![Verilog Full Adder](img/5d0e31451f8b511fe7832605613b6128.png)

**真值表**

| A | B | 宫颈癌前病变 | 标准输出 | 总额 |
| Zero | Zero | Zero | Zero | Zero |
| Zero | Zero | one | Zero | one |
| Zero | one | Zero | Zero | one |
| Zero | one | one | one | Zero |
| one | Zero | Zero | Zero | one |
| one | Zero | one | one | Zero |
| one | one | Zero | one | Zero |
| one | one | one | one | one |

**例**

下面是一个 4 位加法器的例子，它通过信号 a 和 b 接受两个二进制数。

加法器是一种组合电路。因此，Verilog 可以使用连续赋值 ***赋值*** 或 ***始终*** 块建模，该块具有包含所有输入的敏感度列表。

```

module fulladder (  input [3:0] a,
                  input [3:0] b,
                  input c_in,
                  output c_out,
                  output [3:0] sum);

   assign {c_out, sum} = a + b + c_in;
endmodule

```

下面的代码显示了如何使用一个 ***【始终】*** 块，每当它的任何输入改变值时，该块就会被执行。

```

module fulladder (  input [3:0] a,
                  input [3:0] b,
                  input c_in,
                  output reg c_out,
                  output reg [3:0] sum);

	always @ (a or b or c_in) begin
  	{c_out, sum} = a + b + c_in;
  end
endmodule

```

**硬件示意图**

![Verilog Full Adder](img/27e4f77acc10b0aa0e91500ea92f27fa.png)

**试验台**

首先，添加时间刻度指令。它以严重的口音开始，但不以分号结束。时间刻度指令用于指定进一步模块中使用的时间单位和时间分辨率(一皮秒)。时间分辨率是决定模块中时间单位准确度的精度因素。

接下来是模块和变量声明。

*   寄存器(reg)类型保存该值，直到下一个值被其上的时钟脉冲驱动，并且总是在 ***初始*** 或 ***总是*** 块下。它用于对输入施加刺激。
*   为被动变量声明线(wire)。它们的值不会改变，也不能在内部分配给它们，总是一个初始块。

然后是模块实例化。

*   测试台向被测设备(DUT)施加刺激。DUT 必须在测试平台下实例化。端口映射是测试平台模块与设计模块的链接。
*   现在我们给输入变量一个初始刺激。这是在 ***初始*** 块下完成的。
*   我们也可以使用$finish 在前面提到的延迟时间内停止模拟。

不同的是使用了两个系统任务:

*   ***$dumpfile*** 用于将网络和寄存器的值的变化转储到 VCD 文件(值变化转储文件)中。
*   ***$dumpvars*** 用于指定在 file name 参数指定的文件名中应该转储哪些变量。
*   现在就看用户要不要在 TCL 控制台上显示模拟结果了。我们使用了一个 ***$monitor*** ，每当信号值发生变化时，它就会显示信号值。
*   它在 ***【始终】*** 块内执行，灵敏度列表保持与上一节所述相同。
*   格式说明符 ***%t*** 给出了当前的模拟时间， ***%d*** 用于以小数形式显示变量的值。

```

module tb_fulladd;
	// 1\. Declare testbench variables
   reg [3:0] a;
   reg [3:0] b;
   reg c_in;
   wire [3:0] sum;
   integer i;

	// 2\. Instantiate the design and connect to testbench variables
   fulladd  fa0 ( .a (a),
                  .b (b),
                  .c_in (c_in),
                  .c_out (c_out),
                  .sum (sum));

	// 3\. Provide stimulus to test the design
   initial begin
      a <= 0;
      b <= 0;
      c_in <= 0;

      $monitor ("a=0x%0h b=0x%0h c_in=0x%0h c_out=0x%0h sum=0x%0h", a, b, c_in, c_out, sum);

		// Use a for loop to apply random values to the input
      for (i = 0; i < 5; i = i+1) begin
         #10 a <= $random;
             b <= $random;
         		 c_in <= $random;
      end
   end
endmodule

```

当 a 和 b 相加得出一个大于 4 位宽的数字时，总和翻转到零，c_out 变为 1。例如，用黄色突出显示的行加起来就是 0x11，低 4 位分配给 sum，位#4 分配给 c_out。

* * **