# Verilog 优先级编码器

> 原文：<https://www.javatpoint.com/verilog-priority-encoder>

编码器是一种组合电路。它具有 2^n 输入线和 n 条输出线。它接收这些 2^n 输入数据，并将它们编码成 n 位数据。它产生输入线的二进制代码等价物，该输入线为高电平有效。

但是，一个普通的编码器有一个问题。如果有多条逻辑值为 1 的输入线，它将编码错误的输出。它只在只有一个输入为高电平时工作。在多个高输入的情况下，它会出现故障。

因此，为了解决上述缺点，我们“优先考虑”每个输入的级别。因此，如果选择了多条输入线，输出代码将对应于具有最高指定优先级的输入。这种类型的编码器称为 ***优先编码*** r

**例**

```

module pr_en ( input [7:0] a,
               input [7:0] b,
               input [7:0] c,
               input [7:0] d,
               input [1:0] sel,
               output reg [7:0] out);

   always @ (a or b or c or d or sel) begin
      if (sel == 2'b00)
         out <= a;
      else if (sel == 2'b01)
         out <= b;
      else if (sel == 2'b10)
         out <= c;
      else
         out <= d;
   end
endmodule

```

**硬件示意图**

![Verilog Priority Encoder](img/cffc66aef7afa3323c4a995449c102ed.png)

**试验台**

测试台是用于测试另一个模块的高密度脂蛋白模块，称为 ***被测设备(DUT)*** 。测试台包含将输入应用到 DUT 的语句，并检查是否产生了正确的输出。

输入和期望的输出模式被称为*测试向量。以下是优先编码器的测试台:*

 *```

module tb_4to1_mux;
   reg [7:0] a;
   reg [7:0] b;
   reg [7:0] c;
   reg [7:0] d;
   wire [7:0] out;
   reg [1:0] sel;
   integer i;

   pr_en    pr_en0 (   .a (a),
                           .b (b),
                           .c (c),
                           .d (d),
                           .sel (sel),
                           .out (out));

   initial begin
      sel <= 0;
      a <= $random;
      b <= $random;
      c <= $random;
      d <= $random;

      for (i = 1; i < 4; i=i+1) begin
         #5 sel <= i;
      end

      #5 $finish;
   end
endmodule

```

现在，我们将看到如何在 [Verilog](https://www.javatpoint.com/verilog) 中使用不同的建模风格设计一个 4:2 优先级编码器。

### 1.门级建模

这实际上是设计人员用来实现最低级别模块的最低抽象层，因为开关级建模并不常见。顾名思义，门级建模利用了 Verilog 中可用的门原语。

下面我们将描述一个使用门级建模的优先级编码器:

![Verilog Priority Encoder](img/e42955479e12b4ee21190d13c57de375.png)

从电路中，我们可以观察到设计需要一个与门、两个或门和一个非门。让我们开始编码。

**4:2 优先级编码器的门级建模**

和任何 Verilog 代码一样，我们从声明模块和终端端口开始。

```

module priority_encoder_42(A0,A1,Y0,Y1,Y2,Y3);
....
endmodule

```

请注意，我们首先声明输出，然后声明输入，因为内置门也遵循相同的模式。让我们声明输入和输出端口。

```

input Y3, Y2, Y1, Y0;
output A0, A1;

```

现在，我们可以宣布中间信号。这些信号不是终端端口。从上述电路中，来自“非”门和“与”门的信号被视为中间信号。

```

wire y2bar; //not of y2 
wire and_out; // and of y2bar and y1

```

现在我们定义逻辑门。我们使用 gate ( ***<输出>、<输入>*** )语法来使用 Verilog 中内置的 gates。

```

not(y2bar, y2);
and(and_out, y2bar, y1);
or(A1, Y3, Y2);
or(A0, and_out, Y3);

```

因此，我们的最终代码如下:

```

module priority_encoder_42(A0,A1,Y0,Y1,Y2,Y3);
input Y3, Y2, Y1, Y0;
output A0, A1;
wire y2bar;                   //not of y2 
wire and_out;               // and of y2bar and y1
not(y2bar, y2);
and(and_out, y2bar, y1);
or(A1, Y3, Y2);
or(A0, and_out, Y3);
endmodule

```

### 2.数据流建模

在这种建模技术中，我们使用逻辑方程来描述从输入到输出的数据流。我们不需要为组成电路的门而烦恼。

因此，使用这个抽象层次构建复杂的电路要容易得多，因为不需要知道实际的物理布局。

它使用 assign 关键字通过逻辑方程来描述电路。

优先级编码器的逻辑公式为:

```

A1 = Y3 + Y2
A0 = Y3 + Y1Y2?

```

**4:2 优先级编码器的数据流建模**

像往常一样，我们从模块和端口声明开始:

```

module priority_encoder_datafloe(A0,A1,Y0,Y1,Y2,Y3);
input Y0,Y1,Y2,Y3;
output A0,A1;

```

现在，我们必须使用 ***分配*** 来描述输出的数据流。

```

assign A1 = Y3 + Y2;
assign A0 = Y3 + ((~Y2)&Y1);

```

因此，我们的最终代码:

```

module priority_encoder_datafloe(A0,A1,Y0,Y1,Y2,Y3);
input Y0,Y1,Y2,Y3;
output A0,A1;

assign A1 = Y3 + Y2;
assign A0 = Y3 + ((~Y2)&Y1);
endmodule

```

### 3.行为建模

行为建模是 Verilog HDL 中最高层次的抽象。我们只要知道电路是如何工作的，就可以描述它。我们不需要知道逻辑电路或逻辑方程。真值表如下所示，例如:

| Y3 | Y2 | Y1 | Y0 | 一流的 | A0 |
| Zero | Zero | Zero | one | Zero | Zero |
| Zero | Zero | one | X | Zero | one |
| Zero | one | X | X | one | Zero |
| one | X | X | X | one | one |

有了这个真值表，我们可以使用 Verilog 设计我们的优先级编码器。

**4:2 优先级编码器的行为建模**

让我们从模块和端口声明开始。

```

module priority_encoderbehave(A, Y);
input [3:0]Y;
output reg [1:0]A;

```

在行为建模中，我们不得不提到输出为 ***reg*** 。当我们在这个建模风格中使用 ***过程赋值*** 时，我们必须确保输出保持它们的值，直到下一个值被赋予它们。

```

always@(Y)
begin
....
end

```

我们在括号中声明的是敏感度列表。在这里，取决于 ***Y*** 的值。 ***【始终】*** 关键字将确保每次触发敏感度列表时执行语句。

在 ***开始*** 和 ***结束*** 之间，我们编写系统如何工作的程序:

```

casex(Y)
4'b0001:A = 2'b00;
4'b001x:A = 2'b01;
4'b01xx:A = 2'b10;
4'b1xxx:A = 2'b11;
default:$display("Error!");
endcase

```

***案例*** 将一个表达式与一系列案例进行比较，并执行与第一个匹配案例相关联的语句或语句组。我们使用了**T5【casex**，这是 ***case*** 的特殊版本。这将把 x 和 z 值视为无关紧要。

```

module priority_encoderbehave(A, Y);
input [3:0]Y;
output reg [1:0]A;
always@(Y)
begin
casex(Y)
4'b0001:A = 2'b00;
4'b001x:A = 2'b01;
4'b01xx:A = 2'b10;
4'b1xxx:A = 2'b11;
default:$display("Error!");
endcase
end
endmodule

```

### 4.结构建模

结构建模描述了数字系统的硬件结构。这有点类似于门级建模。唯一的区别是它不包括任何内置的门。我们为每个门创建单独的模块，然后集成形成整个电路。

**逻辑电路**

在 4:2 优先级编码器的情况下，我们需要两个或门，一个与门和一个非门。

![Verilog Priority Encoder](img/cb0b8e387033c592207620b0c238eee6.png)

**4:2 优先级编码器的结构建模**

首先从代码开始，我们将首先结构化 OR 门。

我们将 ***模块*** 声明为 or_gate。然后，我们声明输入和输出端口

```

module or_gate(c,a,b);
input a,b
output c;

```

然后，我们用 ***赋值*** 语句来写 or 的逻辑表达式。

```

assign c = a|b;

```

因此，我们的或门模块将是:

```

module or_gate(c,a,b);
input a,b;
output c;
assign c = a|b;
endmodule

```

同样，我们对与门也是如此:

```

module and_gate(z,x,y);
input x,y;
output z;
assign z = x&y;
endmodule 

```

和非门:

```

module not_gate(f,e);
input e;
output f;
assign f = ~e;
endmodule

```

#### 注意:我们在一个模块中保留了不同于其他模块的用于分配输入和输出的变量。它确保模拟过程中不会发生信号混合。

现在，我们可以继续将优先级编码器描述为顶级模块。

像往常一样，从模块和端口声明开始。

```

module priority_encoder_struct(A0,A1,Y0,Y1,Y2,Y3);
input Y0, Y1, Y2, Y3;
output A0,A1;

```

现在将这些单独的逻辑门模块组合成一个单独的顶层模块。它是在模块实例化概念的帮助下完成的，在这个概念中，顶层模块是使用底层模块构建的。

使用逻辑电路，我们将通过端口名 使用 ***实例化这个顶部的较低模块。***

```

not_gate u1(.f(y2bar), .e(y2));
and_gate u2(.z(w1), .x(y2bar), .y(y1));
or_gate(.c(A1), .a(Y3), .b(Y2));
or_gate(.c(A0), .a(Y1), .b(w1));

```

因此，结构形式的优先级编码器的 Verilog 代码为:

```

module or_gate(c,a,b);
input a,b;
output c;
assign c = a|b;
endmodule
module not_gate(f,e);
input e;
output f;
assign f = ~e;
endmodule
module and_gate(z,x,y);
input x,y;
output z;
assign z = x&y;
endmodule 
module priority_encoder_struct(A0,A1,Y0,Y1,Y2,Y3);
input Y0, Y1, Y2, Y3;
output A0,A1;
not_gate u1(.f(y2bar), .e(y2));
and_gate u2(.z(w1), .x(y2bar), .y(y1));
or_gate(.c(A1), .a(Y3), .b(Y2));
or_gate(.c(A0), .a(Y1), .b(w1));
endmodoule

```

* * **